'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createReadModelsReducer;

var _action_types = require('./action_types');

var _diffJson = require('diff-json');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function copyResolveSerials(oldState, newState) {
  Object.getOwnPropertyNames(oldState).forEach(function (key) {
    var propertyDescriptor = Object.getOwnPropertyDescriptor(oldState, key);

    if (key.indexOf('resolve-serial:') !== 0 || propertyDescriptor.enumerable || propertyDescriptor.writable) {
      return;
    }

    Object.defineProperty(newState, key, propertyDescriptor);
  });
}

function createReadModelsReducer() {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    switch (action.type) {
      case _action_types.READMODEL_LOAD_INITIAL_STATE:
        {
          var readModelName = action.readModelName,
              resolverName = action.resolverName,
              initialState = action.initialState,
              serialId = action.serialId;

          var nextState = _extends({}, state, _defineProperty({}, readModelName, _extends({}, state[readModelName] || {}, _defineProperty({}, resolverName, initialState))));

          copyResolveSerials(state, nextState);
          Object.defineProperty(nextState, 'resolve-serial:' + readModelName + ':' + resolverName, {
            configurable: true,
            writable: false,
            enumerable: false,
            value: serialId
          });

          return nextState;
        }

      case _action_types.READMODEL_DROP_STATE:
        {
          var _readModelName = action.readModelName,
              _resolverName = action.resolverName;

          var _nextState = _extends({}, state, _defineProperty({}, _readModelName, _extends({}, state[_readModelName] || {}, _defineProperty({}, _resolverName, null))));

          copyResolveSerials(state, _nextState);
          delete _nextState['resolve-serial:' + _readModelName + ':' + _resolverName];
          delete _nextState[_readModelName][_resolverName];

          return _nextState;
        }

      case _action_types.READMODEL_SUBSCRIPTION_DIFF:
        {
          var _readModelName2 = action.readModelName,
              _resolverName2 = action.resolverName,
              _serialId = action.serialId,
              diff = action.diff;


          if (!state[_readModelName2] || !state[_readModelName2].hasOwnProperty(_resolverName2) || state['resolve-serial:' + _readModelName2 + ':' + _resolverName2] !== _serialId) {
            return state;
          }

          var wrappedResolverState = {
            wrap: state[_readModelName2][_resolverName2]
          };

          (0, _diffJson.applyChanges)(wrappedResolverState, diff);

          var _nextState2 = _extends({}, state, _defineProperty({}, _readModelName2, _extends({}, state[_readModelName2], _defineProperty({}, _resolverName2, wrappedResolverState.wrap))));

          copyResolveSerials(state, _nextState2);

          return _nextState2;
        }

      default:
        return state;
    }
  };
}