'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkRequiredFields = checkRequiredFields;
exports.getRootableUrl = getRootableUrl;
exports.getKey = getKey;
exports.delay = delay;
exports.makeLateResolvingPromise = makeLateResolvingPromise;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function checkRequiredFields(obj, beforeWarnings, afterWarnings) {
  var warningMessages = Object.keys(obj).map(function (fieldName) {
    return obj[fieldName] ? null : 'The \'' + fieldName + '\' field is required';
  }).filter(function (msg) {
    return msg;
  });

  var shouldWarningsBePrinted = warningMessages.length > 0;

  if (shouldWarningsBePrinted) {
    // eslint-disable-next-line no-console
    console.warn([beforeWarnings].concat(_toConsumableArray(warningMessages), [afterWarnings]).filter(function (line) {
      return line;
    }).join('\n'));
  }

  return !shouldWarningsBePrinted;
}

function getRootableUrl(path) {
  var rootDir = typeof process !== 'undefined' && typeof process.env !== 'undefined' && process.env['ROOT_PATH'] ? process.env['ROOT_PATH'] : '';

  var isReactNative = typeof navigator !== 'undefined' && navigator.product === 'ReactNative';

  if (isReactNative && path === '/socket/') {
    rootDir = '';
  }

  return '' + rootDir + path;
}

function getKey(viewModel, aggregateId) {
  return viewModel + ':' + aggregateId;
}

function delay(timeout) {
  return new Promise(function (resolve) {
    return setTimeout(resolve, timeout);
  });
}

function makeLateResolvingPromise() {
  if (arguments.length > 1) {
    throw new Error('Make late-resolved promise accepts only zero on one argument');
  }
  var promiseResolver = function promiseResolver() {
    return null;
  };
  var promise = new Promise(function (resolve) {
    return promiseResolver = resolve;
  });

  var result = function result(arg) {
    return promiseResolver(arg);
  };
  Object.setPrototypeOf(result, Promise.prototype);

  result.then = function (onResolve, onReject) {
    return promise.then(onResolve, onReject);
  };
  result.catch = function (onReject) {
    return promise.catch(onReject);
  };

  if (arguments.length === 1) {
    result(arguments.length <= 0 ? undefined : arguments[0]);
  }

  return result;
}